# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# noqa: D205, D400
"""
Sphinx-Gallery
==============

This example shows how to add a new example when using `Sphinx-Gallery
<https://sphinx-gallery.github.io/>`_.

To use Sphinx-Gallery, first install the package with this command:

.. code-block:: bash

        pip install sphinx-gallery

Then, add the package to the ``extensions`` variable in your Sphinx ``conf.py`` file:

.. code-block:: python

        extensions = [
            "sphinx_gallery.gen_gallery",
        ]
"""
###############################################################################
# Plot a simple sphere using PyVista
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# This code plots a simple sphere using PyVista.

import pyvista as pv

pv.set_jupyter_backend("html")

sphere = pv.Sphere()
sphere.plot()

###############################################################################
# Plot a simple sphere using PyVista with a plotter

plotter = pv.Plotter(notebook=True)
plotter.add_mesh(sphere, color="white", show_edges=True)
plotter.title = "3D Sphere Visualization"
plotter.show()

###############################################################################
# Figures with Matplotlib
# ~~~~~~~~~~~~~~~~~~~~~~~
# This example shows how to render a figure using Matplotlib.

import matplotlib.pyplot as plt
import numpy as np

time = np.linspace(0, 2 * np.pi, 100)

fig, ax = plt.subplots()
ax.plot(time, np.cos(time), color="blue", label=r"$\cos{(t)}$")
ax.plot(time, np.sin(time), color="red", label=r"$\sin{(t)}$")

ax.set_xlabel("Time [time units]")
ax.set_ylabel("Amplitude [distance units]")
ax.set_title("Trigonometric functions")

plt.show()

###############################################################################
# Figures with Plotly
# ~~~~~~~~~~~~~~~~~~~
# This example shows how to render a figure using Plotly.

import plotly.graph_objs as go

# More info: https://plotly.com/python/renderers/

time = np.linspace(0, 2 * np.pi, 100)

cos_trace = go.Scatter(x=time, y=np.cos(time), mode="lines", name="cos(t)")
sin_trace = go.Scatter(x=time, y=np.sin(time), mode="lines", name="sin(t)")

fig = go.Figure(data=[cos_trace, sin_trace])

fig
###############################################################################

import numpy as np
import plotly.express as px

df = px.data.tips()
fig = px.bar(
    df,
    x="sex",
    y="total_bill",
    facet_col="day",
    color="smoker",
    barmode="group",
    template="presentation+plotly",
)
fig.update_layout(height=400)
fig

###############################################################################
# Render equations using IPython ``math``
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# This example shows how to render equations using the IPython ``math`` module.

from IPython.display import Math, display

# LaTeX formatted equation
equation = r"\int\limits_{-\infty}^\infty f(x) \delta(x - x_0) \, dx = f(x_0)"
# Display the equation
display(Math(equation))

###############################################################################
from IPython.display import Latex

Latex(r"This is a \LaTeX{} equation: $a^2 + b^2 = c^2$")

###############################################################################
# Render a table
# ~~~~~~~~~~~~~~
# This is an example to render a table with Sphinx-Gallery.
#
# +--------------------------------+--------------------------------+------------------------------+
# | A                              | B                              | A and B                      |
# +================================+================================+==============================+
# | False                          | False                          | False                        |
# +--------------------------------+--------------------------------+------------------------------+
# | True                           | False                          | False                        |
# +--------------------------------+--------------------------------+------------------------------+
# | False                          | True                           | False                        |
# +--------------------------------+--------------------------------+------------------------------+
# | True                           | True                           | True                         |
# +--------------------------------+--------------------------------+------------------------------+
#

###############################################################################
# Render a table using pandas

import pandas as pd

# Create a dictionary of data
data = {
    "A": [True, False, True, False],
    "B": [False, True, False, True],
    "C": [True, True, False, False],
}

# Create DataFrame from the dictionary
df = pd.DataFrame(data)

# Display the DataFrame
df.head()

###############################################################################

# %%



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module to configure cheatsheet options and build the Quarto cheatsheet.

This module provides functions to configure cheatsheet options, build the Quarto cheatsheet,
and add the cheatsheet to the left navigation sidebar.
"""

import pathlib
import subprocess
from typing import List, Optional

from sphinx.application import Sphinx
from sphinx.util import logging

logger = logging.getLogger(__name__)

# Cheat sheet extension version
CHEAT_SHEET_QUARTO_EXTENTION_VERSION = "v1"


def cheatsheet_sidebar_pages(app: Sphinx) -> Optional[List[str]]:
    """
    Get the pages to display the cheat sheet sidebar and return the list of pages.

    Parameters
    ----------
    app : sphinx.application.Sphinx
        Application instance for rendering the documentation.

    Returns
    -------
    Optional[List[str]]
        List of pages to display the cheat sheet sidebar, or None if not configured.
    """
    html_theme_options = app.config.html_theme_options
    cheatsheet_options = html_theme_options.get("cheatsheet")
    if not cheatsheet_options:
        return None
    pages = cheatsheet_options.get("pages", ["index"])
    cheatsheet_pages = [pages] if isinstance(pages, str) else pages
    return cheatsheet_pages


def convert_pdf_to_png(pdf_path: pathlib.Path, output_dir: pathlib.Path, output_png: str) -> None:
    """
    Convert PDF to PNG images.

    Parameters
    ----------
    pdf_path : pathlib.Path
        Path to the PDF file.
    output_dir : pathlib.Path
        Path to the output directory.
    output_png : str
        Name of the output PNG file.
    """
    try:
        from pdf2image import convert_from_path
    except ImportError as e:
        raise ImportError(
            f"Failed to import `pdf2image`: {e}. Install the package using `pip install pdf2image`"
        )
    try:
        images = convert_from_path(pdf_path, 500)
        images[0].save(output_dir / output_png, "PNG")
    except Exception as e:
        raise RuntimeError(
            f"Failed to convert PDF to PNG: {e}. Ensure the PDF file is valid and poppler is installed."  # noqa:E501
        )


def run_quarto_command(command: List[str], cwd: str) -> None:
    """
    Run a Quarto command and log its output.

    Parameters
    ----------
    command : List[str]
        List of command arguments.
    cwd : str
        Current working directory.
    """
    command = ["quarto"] + command
    try:
        result = subprocess.run(command, cwd=cwd, check=True, capture_output=True, text=True)
        if result.stdout:
            logger.info(result.stdout)

        if result.stderr:
            # HACK: Quarto writes both stdout and stderr to stderr
            # so we need to log it as info if it's not an error
            logger.info(result.stderr)

    except subprocess.CalledProcessError as e:
        raise RuntimeError(f"Failed to run the command: {e}")


def build_quarto_cheatsheet(app: Sphinx) -> None:
    """
    Build the Quarto cheatsheet.

    Parameters
    ----------
    app : sphinx.application.Sphinx
        Application instance for rendering the documentation.
    """
    cheatsheet_options = app.config.html_theme_options.get("cheatsheet", {})

    if not cheatsheet_options:
        return

    cheatsheet_file = cheatsheet_options.get("file", "")
    if not cheatsheet_file:
        return

    output_dir = "_static"
    version = cheatsheet_options.get("version", "main")

    cheatsheet_file = pathlib.Path(app.srcdir) / cheatsheet_file
    output_dir_path = pathlib.Path(app.outdir) / output_dir
    file_name = str(cheatsheet_file.name)
    file_path = cheatsheet_file.parent

    logger.info(f"Building Quarto cheatsheet: {file_name}")

    # Adapt with new
    run_quarto_command(["--version"], file_path)
    run_quarto_command(
        [
            "add",
            f"ansys/pyansys-quarto-cheatsheet@{CHEAT_SHEET_QUARTO_EXTENTION_VERSION}",
            "--no-prompt",
        ],
        file_path,
    )
    run_quarto_command(
        [
            "render",
            file_name,
            "--to",
            "cheat_sheet-pdf",
            "--output-dir",
            output_dir_path,
            "-V",
            f"version={version}",
        ],
        file_path,
    )
    run_quarto_command(
        ["remove", "ansys/pyansys-quarto-cheatsheet", "--no-prompt"],
        file_path,
    )
    supplementary_files = [
        "_static/slash.png",
        "_static/bground.png",
        "_static/ansys.png",
    ]
    for file in supplementary_files:
        file_path = cheatsheet_file.parent / file
        if file_path.exists():
            file_path.unlink()

    # If static folder is clean, delete it
    if not list(cheatsheet_file.parent.glob("_static/*")):
        cheatsheet_file.parent.joinpath("_static").rmdir()

    output_file = output_dir_path / file_name.replace(".qmd", ".pdf")
    app.config.html_theme_options["cheatsheet"]["output_dir"] = f"{output_dir}/{output_file.name}"
    output_png = file_name.replace(".qmd", ".png")
    # Check output file exists
    if not output_file.exists():
        raise FileNotFoundError(f"Failed to build Quarto cheatsheet: {output_file} does not exist.")

    convert_pdf_to_png(output_file, output_dir_path, output_png)
    logger.info(f"Cheat sheet build finished successfully: {output_file}")
    app.config.html_theme_options["cheatsheet"]["thumbnail"] = f"{output_dir}/{output_png}"



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Initiate the search."""

from sphinx.application import Sphinx

from ansys_sphinx_theme.search.fuse_search import (
    create_search_index,
)


def update_search_config(app: Sphinx) -> None:
    """Update the search configuration.

    Parameters
    ----------
    app : Sphinx
        Sphinx application.
    """
    theme_static_options = app.config.html_theme_options.get("static_search", {})
    theme_static_options["keys"] = ["title", "text", "objectID"]
    theme_static_options["threshold"] = theme_static_options.get("threshold", 0.2)
    theme_static_options["limit"] = theme_static_options.get("limit", 10)
    app.add_config_value("index_patterns", {}, "html")
    app.config.html_theme_options["static_search"] = theme_static_options


__all__ = [
    "create_search_index",
    "update_search_config",
]



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module to configure whatsnew options and layer the what's new content on the documentation.

This module provides functions to configure whatsnew options, extract the what's new content
from the changelog document, and layer the what's new content on the documentation.
"""

import pathlib
import re
from typing import Iterable

from docutils import nodes
from sphinx.application import Sphinx

"""Semantic version regex as found on semver.org:
https://semver.org/#is-there-a-suggested-regular-expression-regex-to-check-a-semver-string"""
SEMVER_REGEX = (
    r"(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)"
    r"(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)"
    r"(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?"
    r"(?:\+(>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?"
)


def get_whatsnew_options(app: Sphinx) -> tuple:
    """Get the whatsnew options from the configuration file.

    Parameters
    ----------
    app : sphinx.application.Sphinx
        Application instance for rendering the documentation.

    Returns
    -------
    tuple
        Tuple containing the paths to the whatsnew file, changelog file, and the sidebar pages.
        If the whatsnew options are not found, return None for each of those fields.
    """
    # Get the html_theme_options from conf.py
    config_options = app.config.html_theme_options

    # Get the whatsnew key from the html_theme_options
    whatsnew_options = config_options.get("whatsnew")

    if not whatsnew_options:
        return None, None

    # Get the names of the whatsnew.yml and changelog.rst files
    whatsnew_file = whatsnew_options.get("whatsnew_file_name")
    changelog_file = whatsnew_options.get("changelog_file_name")

    # The source directory of the documentation: {repository_root}/doc/source
    doc_src_dir = app.env.srcdir

    if whatsnew_file is not None:
        whatsnew_file = pathlib.Path(doc_src_dir) / whatsnew_file
    if changelog_file is not None:
        changelog_file = pathlib.Path(doc_src_dir) / changelog_file

    return whatsnew_file, changelog_file


def add_whatsnew_changelog(app: Sphinx, doctree: nodes.document) -> None:
    """Add the what's new section to each minor version if applicable.

    Parameters
    ----------
    app : sphinx.application.Sphinx
        Application instance for rendering the documentation.
    doctree : docutils.nodes.document
        Document tree for the page.
    """
    # Get the html_theme_options from conf.py
    config_options = app.config.html_theme_options

    # Get the whatsnew key from the html_theme_options
    whatsnew_options = config_options.get("whatsnew")

    # The source directory of the documentation: {repository_root}/doc/source
    doc_src_dir = pathlib.Path(app.env.srcdir)

    # Full paths to the whatsnew.yml and changelog.rst files from the doc/source directory
    whatsnew_file = doc_src_dir / whatsnew_options.get("whatsnew_file_name")
    changelog_file = doc_src_dir / whatsnew_options.get("changelog_file_name")

    # Read the file and get the sections from the file as a list. For example,
    # sections = [<document: <target...><section "getting started; ref-getting-starte ...>]
    sections = doctree.traverse(nodes.document)
    if not sections:
        return

    # Get the file name of the section using section.get("source") and return the section
    # if section.get("source") is equal to the changelog_file
    changelog_doctree_sections = [
        section for section in sections if section.get("source") == str(changelog_file)
    ]

    # Return if the changelog file sections are not found
    if not changelog_doctree_sections:
        return

    # Get the what's new data from the whatsnew.yml file
    whatsnew_data = get_whatsnew_data(whatsnew_file)

    existing_minor_versions = []
    docs_content = doctree.traverse(nodes.section)

    # Get each section that contains a semantic version number
    version_sections = [node for node in docs_content if re.search(SEMVER_REGEX, node[0].astext())]

    for node in version_sections:
        # Get the semantic version number from the section title link
        next_node = node.next_node(nodes.reference)
        # Get the name of the section title link
        version = next_node.get("name")

        if version:
            # Create the minor version from the patch version
            minor_version = ".".join(version.split(".")[:-1])

            if minor_version not in existing_minor_versions:
                # Add minor version to list of existing minor versions
                existing_minor_versions.append(minor_version)

                # Create a section for the minor version
                minor_version_section = nodes.section(
                    ids=[f"version-{minor_version}"], names=[f"Version {minor_version}"]
                )
                # Add the title to the section for the minor version
                minor_version_section += nodes.title("", f"Version {minor_version}")

                # Add "What's New" section under the minor version if the minor version is in
                # the what's new data
                if whatsnew_file.exists() and (minor_version in list(whatsnew_data.keys())):
                    minor_version_whatsnew = add_whatsnew_section(minor_version, whatsnew_data)
                    minor_version_section.append(minor_version_whatsnew)

                # Add the title at the beginning of a section with a patch version
                node.insert(0, minor_version_section)


def get_whatsnew_data(whatsnew_file: pathlib.Path) -> dict:
    """Get the what's new data from the whatsnew.yml file.

    Parameters
    ----------
    whatsnew_file : pathlib.Path
        Path to the whatsnew.yml file.

    Returns
    -------
    dict
        Dictionary containing the what's new data from the whatsnew.yml file.
    """
    if whatsnew_file.exists():
        # Open and read the whatsnew.yml file

        try:
            import yaml
        except ImportError as e:
            raise ImportError(
                f"Failed to import `pyyaml`: {e}. Install the package using `pip install ansys-sphinx-theme[changelog]`"  # noqa: E501
            )

        with pathlib.Path.open(whatsnew_file, "r", encoding="utf-8") as file:
            whatsnew_data = yaml.safe_load(file)

        # Create a dictionary containing the what's new data for each minor version
        # For example: { minor_version: [fragment1_dict, fragment2_dict, ...] }
        minor_version_whatsnew_data = {}
        for fragment in whatsnew_data["fragments"]:
            # Get the minor version from the fragment version
            whatsnew_minor_version = ".".join(fragment["version"].split(".")[:2])

            # Create an empty list for the minor version if it does not exist
            if whatsnew_minor_version not in minor_version_whatsnew_data:
                minor_version_whatsnew_data[whatsnew_minor_version] = []
            # Append the fragment to the minor version in the whatsnew_data
            minor_version_whatsnew_data[whatsnew_minor_version].append(fragment)

        return minor_version_whatsnew_data


def add_whatsnew_section(minor_version: str, whatsnew_data: dict) -> nodes.section:
    """Add the what's new section and dropdowns for each fragment in the whatsnew.yml file.

    Parameters
    ----------
    minor_version : str
        Minor version number.
    whatsnew_data : dict
        Dictionary containing the what's new data from the whatsnew.yml file.

    Returns
    -------
    nodes.section
        Section containing the what's new title and dropdowns for each fragment in the
        whatsnew.yml file.
    """
    # Add the what's new section and title
    minor_version_whatsnew = nodes.section(ids=[f"version-{minor_version}-whatsnew"])
    minor_version_whatsnew += nodes.title("", "What's New")

    # Add a dropdown under the "What's New" section for each fragment in the whatsnew.yml file
    for fragment in whatsnew_data[minor_version]:
        # Create a dropdown for the fragment
        whatsnew_dropdown = nodes.container(
            body_classes=[""],
            chevron=True,
            container_classes=["sd-mb-3 sd-fade-in-slide-down"],
            design_component="dropdown",
            has_title=True,
            icon="",
            is_div=True,
            opened=False,
            title_classes=[""],
            type="dropdown",
        )

        # Set the title_id for the dropdown
        title_id = fragment["title"].replace(" ", "-").lower()

        # Add the title of the fragment to the dropdown
        whatsnew_dropdown += nodes.rubric(ids=[title_id], text=fragment["title"])

        # Add a line specifying which version the fragment is available in
        version_paragraph = nodes.paragraph("sd-card-text")
        version_paragraph.append(
            nodes.emphasis("", f"Available in v{fragment['version']} and later")
        )
        whatsnew_dropdown += version_paragraph

        # Split content from YAML file into list
        content_lines = fragment["content"].split("\n")

        # Create iterator for the content_lines
        content_iterator = iter(content_lines)

        # Navigate to first line in the iterator
        line = next(content_iterator, None)

        while line is not None:
            if ".. code" in line or ".. sourcecode" in line:
                # Get language after "code::"
                language = line.split("::")[1].strip()
                # Create the code block container node with the language if it exists
                code_block = (
                    nodes.container(classes=[f"highlight-{language} notranslate"])
                    if language
                    else nodes.container()
                )

                # Fill the code block with the following lines until it reaches the end or an
                # unindented line
                code_block, line = fill_code_block(content_iterator, code_block)
                whatsnew_dropdown += code_block
            else:
                # Create the paragraph node
                paragraph = nodes.paragraph("sd-card-text")

                # Fill the paragraph node with the following lines until it reaches
                # the end or a code block
                paragraph, line = fill_paragraph(content_iterator, paragraph, line)
                whatsnew_dropdown += paragraph

        # Append the fragment dropdown to the minor_version_whatsnew section
        minor_version_whatsnew.append(whatsnew_dropdown)

    return minor_version_whatsnew


def fill_code_block(content_iterator: Iterable, code_block: nodes.container) -> nodes.container:
    """Fill the code block.

    Parameters
    ----------
    content_iterator : Iterable
        Iterator for the content lines from the fragments in the whatsnew.yml file.
    code_block : nodes.container
        Container node for the code block.

    Returns
    -------
    nodes.container, str
        Container node for the code block and the next line in the content iterator.
    """
    # classes=["highlight"] is required for the copy button to show up in the literal_block
    highlight_container = nodes.container(classes=["highlight"])

    # Create literal block with copy button
    literal_block = nodes.literal_block(
        classes=["sd-button sd-button--icon sd-button--icon-only sd-button--icon-small"],
        icon="copy",
        label="Copy",
        title="Copy",
    )

    # Move to the first line in the code block (the line after ".. code::")
    next_line = next(content_iterator, None)

    # Boolean to check if the line in code block is within a dictionary
    in_dictionary = False

    # While the next_line is indented or blank, add it to the code block
    while next_line is not None and (next_line.startswith(" ") or (next_line == "")):
        if in_dictionary:
            if next_line.lstrip().startswith("}"):
                in_dictionary = False
                formatted_line = next_line.lstrip() + "\n"
            else:
                formatted_line = next_line + "\n"
        else:
            if next_line.lstrip().startswith("{"):
                in_dictionary = True
            formatted_line = next_line.lstrip() + "\n"

        # Add the formatted line to the literal block
        literal_block += nodes.inline(text=formatted_line)

        # Break the loop if the end of the content is reached
        if next_line is not None:
            # Move to the next line in the content
            next_line = next(content_iterator, None)
        else:
            break

    # Add the literal block to the highlight container
    highlight_container += literal_block

    # Add the highlight container to the code block
    code_block += highlight_container

    return code_block, next_line


def fill_paragraph(
    content_iterator: Iterable, paragraph: nodes.paragraph, next_line: str
) -> nodes.paragraph:
    """Fill the paragraph node.

    Parameters
    ----------
    content_iterator : Iterable
        Iterator for the content lines from the fragments in the whatsnew.yml file.
    paragraph : nodes.paragraph
        Paragraph node.
    next_line : str
        Next line in the content iterator.

    Returns
    -------
    nodes.paragraph, str
        Paragraph node and the next line in the content iterator.
    """
    # While the next_line is not None and is not a code block, add it to the paragraph
    while next_line is not None and not next_line.startswith(".. "):
        # Regular expressions to find rst links, and single & double backticks/asterisks
        rst_link_regex = r"(`[^<`]+? <[^>`]+?>`_)"
        single_backtick_regex = r"(`[^`]+?`)"
        double_backtick_regex = r"(``.*?``)"
        bold_text_regex = r"(\*\*.*?\*\*)"
        italic_text_regex = r"(\*[^\*]+?\*)"

        # Check if there are rst links, single & double backticks/asterisks in the line
        link_backtick_regex = (
            rf"{rst_link_regex}|"
            rf"{single_backtick_regex}|{double_backtick_regex}|"
            rf"{bold_text_regex}|{italic_text_regex}"
        )

        # Get all matches for rst links, single & double backticks/asterisks in the line
        # Sample: next_line = "The files are **deleted** when the ``GUI`` is closed. For more info"
        # For example, matches = [('', '', '', '**deleted**', ''), ('', '', '``GUI``', '', '')]
        matches = re.findall(link_backtick_regex, next_line)

        if matches:
            # Get all of the matches from the matches list
            # For example, regex_matches = ['**deleted**', '``GUI``']
            regex_matches = [
                element for match in matches for i, element in enumerate(match) if element
            ]

            # Create a regular expression pattern that matches any URL
            # For example, pattern = r"\*\*deleted\*\*|``GUI``"
            pattern = "|".join(map(re.escape, regex_matches))

            # Split the line using the pattern
            # For example, split_lines = ['The files are ', '**deleted**', ' when the ', '``GUI``',
            # ' is closed. For more info']
            split_lines = re.split(f"({pattern})", next_line)

            for line in split_lines:
                if line in regex_matches:
                    # If it matches RST link regex, append a reference node
                    if re.search(rst_link_regex, line):
                        text, url = re.findall(r"`([^<`]+?) <([^>`]+?)>`_", line)[0]
                        if url.startswith("http") or url.startswith("www"):
                            ref_type = "external"
                        else:
                            ref_type = "internal"
                        paragraph.append(
                            nodes.reference(
                                classes=[f"reference-{ref_type}"],
                                refuri=url,
                                href=url,
                                text=text,
                            )
                        )
                    # If it matches single or double backticks, append a literal node
                    elif re.search(single_backtick_regex, line):
                        text = re.findall(r"`([^`]+?)`", line)[0]
                        paragraph.append(nodes.literal(text=text))
                    elif re.search(double_backtick_regex, line):
                        text = re.findall(r"``(.*?)``", line)[0]
                        paragraph.append(nodes.literal(text=text))
                    # If it matches bold text, append a strong node
                    elif re.search(bold_text_regex, line):
                        text = re.findall(r"\*\*(.*?)\*\*", line)[0]
                        paragraph.append(nodes.strong(text=text))
                    # If it matches italic text, append an emphasis node
                    elif re.search(italic_text_regex, line):
                        text = re.findall(r"\*([^\*]+?)\*", line)[0]
                        paragraph.append(nodes.emphasis(text=text))
                else:
                    paragraph.append(nodes.inline(text=line))
        else:
            # Append the next_line as an inline element, unless it is an empty string. If it's an
            # empty string, append a line break
            paragraph.append(nodes.inline(text=next_line)) if next_line != "" else paragraph.append(
                nodes.line(text="\n")
            )

        # Add a space at the end of each line
        paragraph.append(nodes.inline(text=" "))

        # Break the loop if the end of the content is reached
        if next_line is not None:
            # Move to the next line in the content
            next_line = next(content_iterator, None)
        else:
            break

    return paragraph, next_line


def extract_whatsnew(app: Sphinx, doctree: nodes.document, docname: str) -> None:
    """Extract the what's new content from the changelog document.

    Parameters
    ----------
    app : sphinx.application.Sphinx
        Application instance for rendering the documentation.
    doctree : docutils.nodes.document
        Document tree for the page.
    docname : str
        Name of the document being processed.
    """
    # Get the html_theme_options from conf.py
    config_options = app.config.html_theme_options

    # Get the whatsnew key from the html_theme_options
    whatsnew_options = config_options.get("whatsnew")

    # The source directory of the documentation: {repository_root}/doc/source
    doc_src_dir = pathlib.Path(app.env.srcdir)

    # Full path to the changelog.rst file starting from the doc/source directory
    changelog_file = doc_src_dir / whatsnew_options.get("changelog_file_name")

    # Get the number of headers to display in the what's new section in the sidebar
    # By default, it displays the first three minor versions
    sidebar_no_of_headers = whatsnew_options.get("sidebar_no_of_headers", 3)
    # Get the number of what's new content to display under each minor version in the sidebar.
    # By default, it displays all what's new dropdown titles
    sidebar_no_of_contents = whatsnew_options.get("sidebar_no_of_contents")

    # Get the doctree for the file
    changelog_file = changelog_file.stem
    doctree = app.env.get_doctree(changelog_file)
    docs_content = doctree.traverse(nodes.section)
    html_title = app.config.html_title or app.config.html_short_title or app.config.project

    if not docs_content:
        return

    whatsnew = []
    app.env.whatsnew = []

    # Get a list of nodes whose ids start with "version" that contain "What's new" sections
    versions_nodes = []

    # Find nodes that contain the minor versions and a "What's New" section
    for node in docs_content:
        node_id = node.get("ids")[0]
        # Get the nodes that contain the minor versions: "Version x.y"
        if node_id.startswith("version") and "whatsnew" not in node_id:
            sections = list(node.traverse(nodes.section))
            # If the section contains a "What's New" section, add it to the list of versions
            whatsnew_nodes = [
                section_node
                for section_node in sections
                if section_node.get("ids")[0] == f"{node_id}-whatsnew"
            ]
            if whatsnew_nodes:
                versions_nodes.append(node)

    # Get the version nodes up to the specified number of headers
    versions_nodes = versions_nodes[:sidebar_no_of_headers]

    for version_node in versions_nodes:
        # Get the version title (e.g., "Version 0.1")
        version_title = version_node[0].astext()
        # Get the sections under the version node
        sections = list(version_node.traverse(nodes.section))

        # Sections with text that contains "what's new"
        whatsnew_nodes = [node for node in sections if node[0].astext().lower() == "what's new"]

        if not whatsnew_nodes:
            continue

        # Get the children of the "What's New" section
        children = [node for node in whatsnew_nodes[0].traverse(nodes.rubric)]

        # Filter the displayed children based on the number of content specified in the config
        if sidebar_no_of_contents is not None:
            if len(children) > sidebar_no_of_contents:
                children = children[:sidebar_no_of_contents]

        contents = {
            "title": f"{html_title} {version_title}",
            "title_url": f"{changelog_file}.html#{version_node.get('ids')[0]}",
            "children": children,
            "url": f"{changelog_file}.html#{whatsnew_nodes[0]['ids'][0]}",
        }

        whatsnew.append(contents)

    app.env.whatsnew = whatsnew


def whatsnew_sidebar_pages(app: Sphinx) -> list:
    """Get the pages the what's new section should be displayed on and return the list of pages.

    Parameters
    ----------
    app : sphinx.application.Sphinx
        Application instance for rendering the documentation.

    Returns
    -------
    list
        List of pages the what's new section should be displayed on.
    """
    # Get the html_theme_options from conf.py
    config_options = app.config.html_theme_options

    # Get the whatsnew key from the html_theme_options
    whatsnew_options = config_options.get("whatsnew")

    if not whatsnew_options:
        return None

    # Get the pages the whatsnew section should be displayed on
    whatsnew_sidebar_pages = whatsnew_options.get("sidebar_pages")

    return whatsnew_sidebar_pages



"""Sphinx documentation configuration file."""

from datetime import datetime
import os
from pathlib import Path
import subprocess
from typing import Dict, List

from github import Github
import plotly.io as pio
import pyvista
import requests
from sphinx.application import Sphinx
from sphinx.builders.latex import LaTeXBuilder

from ansys_sphinx_theme import (
    __version__,
    ansys_favicon,
    ansys_logo_white,
    ansys_logo_white_cropped,
    generate_404,
    get_version_match,
    latex,
    watermark,
)

pio.renderers.default = "sphinx_gallery"
THIS_PATH = Path(__file__).parent.resolve()
PYANSYS_LIGHT_SQUARE = (THIS_PATH / "_static" / "pyansys_light_square.png").resolve()
EXAMPLE_PATH = (THIS_PATH / "examples" / "sphinx_examples").resolve()

# Project information
project = "ansys_sphinx_theme"
copyright = f"(c) {datetime.now().year} ANSYS, Inc. All rights reserved"
author = "ANSYS, Inc."
release = version = __version__
cname = os.getenv("DOCUMENTATION_CNAME", "sphinxdocs.ansys.com")
switcher_version = get_version_match(__version__)

# HTML configuration
html_favicon = ansys_favicon
html_theme = "ansys_sphinx_theme"
html_short_title = html_title = "Ansys Sphinx Theme"
html_static_path = ["_static"]
templates_path = ["_templates"]

html_context = {
    "github_user": "ansys",
    "github_repo": "ansys-sphinx-theme",
    "github_version": "main",
    "doc_path": "doc/source",
    "page_assets": {
        "examples/table": {
            "needs_datatables": True,
        },
    },
}

html_theme_options = {
    "github_url": "https://github.com/ansys/ansys-sphinx-theme",
    "contact_mail": "pyansys.support@ansys.com",
    "use_edit_page_button": True,
    "additional_breadcrumbs": [
        ("PyAnsys", "https://docs.pyansys.com/"),
    ],
    "switcher": {
        "json_url": f"https://{cname}/versions.json",
        "version_match": get_version_match(__version__),
    },
    "logo": "ansys",
    "search_extra_sources": {
        "PyAnsys": "https://docs.pyansys.com/version/stable/",
        "Actions": "https://actions.docs.ansys.com/version/stable/",
    },
    "search_filters": {
        "User Guide": [
            "user-guide/",
            "getting-started/",
            "index/",
        ],
        "Release Notes": ["changelog"],
        "Examples": ["examples/"],
        "Contributing": ["contribute/"],
    },
}


html_js_files = ["https://cdn.plot.ly/plotly-3.0.1.min.js"]


# Sphinx extensions
extensions = [
    "numpydoc",
    "sphinx_design",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "sphinx_copybutton",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "notfound.extension",
    "sphinx_jinja",
]

# Intersphinx mapping
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "sphinx": ("https://www.sphinx-doc.org/en/master", None),
}

# numpydoc configuration
numpydoc_show_class_members = False
numpydoc_xref_param_type = True
numpydoc_validate = True
numpydoc_validation_checks = {
    "GL06",  # Found unknown section
    "GL07",  # Sections are in the wrong order.
    "GL09",  # Deprecation warning should precede extended summary
    "GL10",  # reST directives {directives} must be followed by two colons
    "SS01",  # No summary found
    "SS02",  # Summary does not start with a capital letter
    "SS04",  # Summary contains heading whitespaces
    "RT02",  # The first line of the Returns section should contain only the type
}

suppress_warnings = ["config.cache"]

# The suffix(es) of source filenames.
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# additional logos for the latex coverpage
LaTeXBuilder.supported_image_types = ["image/png", "image/pdf", "image/svg+xml"]
latex_additional_files = [watermark, ansys_logo_white, ansys_logo_white_cropped]
latex_elements = {"preamble": latex.generate_preamble(html_title)}

# Not found page
notfound_context = {
    "body": generate_404(),
}
notfound_no_urls_prefix = True

# ONLY FOR ANSYS-SPHINX-THEME
exclude_patterns = [
    "links.rst",
    "examples/sphinx-gallery/README.rst",
    "sg_execution_times.rst",
    "examples/gallery-examples/*.ipynb",
]
rst_epilog = ""
with Path.open(THIS_PATH / "links.rst", "r") as f:
    rst_epilog += f.read()


linkcheck_ignore = [
    r"https://sphinxdocs.ansys.com/version/*",
]
if switcher_version != "dev":
    linkcheck_ignore.append(
        f"https://github.com/ansys/ansys-sphinx-theme/releases/tag/v{__version__}"
    )


# Configure the Jinja contexts

jinja_contexts = {
    "install_guide": {
        "version": f"v{version}" if not version.endswith("dev0") else "main",
    },
    "pdf_guide": {"version": get_version_match(__version__)},  # noqa: E501
    "toxenvs": {
        "envs": subprocess.run(
            ["tox", "list", "-d", "-q"], capture_output=True, text=True
        ).stdout.splitlines()[1:],
    },
}


def extract_example_links(
    repo_fullname: str, path_relative_to_root: str, exclude_files: List[str] = []
) -> List[str]:
    """
    Extract example links from a specific GitHub repository.

    Parameters
    ----------
    repo_fullname : str
        Fullname of the repository to extract example links from.
    path_relative_to_root : str
        Path relative to the root of the repository to extract example links from.
    exclude_files : list of str
        A list of files to exclude from the returned example links.

    Returns
    -------
    list
        List of example links.
    """
    g = Github()
    repo = g.get_repo(repo_fullname)
    contents = repo.get_contents(path_relative_to_root)
    if not isinstance(contents, list):
        contents = [contents]
    example_links = []
    for content in contents:
        if content.type == "dir":
            example_links.extend(extract_example_links(repo_fullname, content.path, exclude_files))
        elif content.type == "file":
            if content.name not in exclude_files:
                example_links.append(content.download_url)

    return example_links


def download_and_process_files(example_links: List[str]) -> List[str]:
    """Download and process a series of example files.

    This function downloads a series of example files using a
    list of links and processes each file.

    Parameters
    ----------
    example_links : List[str]
        List of links to the example files to be downloaded.

    Returns
    -------
    list
        List of the names of the processed files.
    """
    file_names = []
    for link in example_links:
        file_name = link.split("/")[-1]
        file_path = str((EXAMPLE_PATH / file_name).absolute())
        with open(file_path, "wb") as f:  # noqa: PTH123
            response = requests.get(link)
            content = response.content.decode()
            lines = content.splitlines()
            # Replace the "target" string with the file name in the example files
            f.write("\n".join([line.replace("target", file_name) for line in lines]).encode())
        file_names.append(file_name)

    return file_names


# Skip building examples if desired
BUILD_EXAMPLES = True if os.environ.get("BUILD_EXAMPLES", "true") == "true" else False
jinja_contexts["main_toctree"] = {"build_examples": BUILD_EXAMPLES}

if not BUILD_EXAMPLES:
    exclude_patterns.extend(["examples.rst", "examples/**", "examples/api/**"])
else:
    # Autoapi examples
    extensions.append("ansys_sphinx_theme.extension.autoapi")
    html_theme_options["ansys_sphinx_theme_autoapi"] = {
        "project": project,
        "directory": "src/ansys_sphinx_theme/examples",
        "output": "examples/api",
        "own_page_level": "function",
        "package_depth": 1,
    }

    # Gallery of examples
    extensions.extend(["nbsphinx", "sphinx_gallery.gen_gallery"])
    sphinx_gallery_conf = {
        # path to your examples scripts
        "examples_dirs": ["examples/sphinx-gallery/"],
        # path where to save gallery generated examples
        "gallery_dirs": ["examples/gallery-examples/"],
        # Pattern to search for example files
        "filename_pattern": r"\.py",
        # Remove the "Download all examples" button from the top level gallery
        "download_all_examples": False,
        # Modules for which function level galleries are created.  In
        "image_scrapers": ("pyvista", "matplotlib", "plotly.io._sg_scraper.plotly_sg_scraper"),
        "default_thumb_file": str(PYANSYS_LIGHT_SQUARE),
    }
    pyvista.BUILDING_GALLERY = True
    pyvista.OFF_SCREEN = True

    nbsphinx_prolog = """
Download this example as a :download:`Jupyter notebook </{{ env.docname }}.ipynb>`.

----
"""
    nbsphinx_thumbnails = {
        "examples/nbsphinx/jupyter-notebook": "_static/pyansys_light_square.png",
    }

    # Third party examples
    example_links = extract_example_links(
        "executablebooks/sphinx-design",
        "docs/snippets/rst",
        exclude_files=["article-info.txt"],
    )
    file_names = download_and_process_files(example_links)

    admonitions_links = extract_example_links(
        "pydata/pydata-sphinx-theme",
        "docs/examples/kitchen-sink/admonitions.rst",
    )

    admonitions_links = download_and_process_files(admonitions_links)
    todo_include_todos = True  # admonition todo needs this to be True

    jinja_contexts["examples"] = {"inputs_examples": file_names}
    jinja_contexts["admonitions"] = {"inputs_admonitions": admonitions_links}


jinja_globals = {
    "ANSYS_SPHINX_THEME_VERSION": version,
}


def revert_exclude_patterns(app, env):
    """Revert the exclude patterns.

    Parameters
    ----------
    app : Sphinx
        Sphinx application instance.
    env : BuildEnvironment
        The build environment.

    Notes
    -----
    Remove the examples/gallery-examples/*.ipynb pattern from the exclude patterns.
    When the nbsphinx extension is enabled, the exclude patterns are modified
    to exclude the examples/gallery-examples/*.ipynb pattern. This function reverts
    the exclude patterns to their original state.
    """
    excluded_pattern = env.config.exclude_patterns
    excluded_pattern.remove("examples/gallery-examples/*.ipynb")
    env.config.exclude_patterns = excluded_pattern


def setup(app: Sphinx) -> Dict:
    """Sphinx hooks to add to the setup."""
    app.connect("env-updated", revert_exclude_patterns)



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for generating search indices."""

import json
from pathlib import Path
import re

from docutils import nodes
from sphinx.util import logging

logger = logging.getLogger(__name__)


class SearchIndex:
    """Generate a search index for a Sphinx document."""

    def __init__(self, doc_name, app, filter_options=None):
        """
        Initialize the search index object.

        Parameters
        ----------
        doc_name : str
            Name of the document.
        app : Sphinx
            Sphinx application instance.
        """
        self.doc_name = doc_name
        self.doc_path = f"{self.doc_name}.html"
        self.env = app.env
        self.theme_options = app.config.html_theme_options.get("static_search", {})
        self.doc_title = self.env.titles[self.doc_name].astext()
        self.doc_tree = self.env.get_doctree(self.doc_name)
        first_part = self.doc_name.split("/")[0]
        try:
            self.parent_title = (
                self.env.titles.get(first_part).astext() if doc_name != "index" else "Home"
            )
        except:  # noqa: E722
            self.parent_title = "Home"
        self.sections = []
        self.filter_options = filter_options

    def build_sections(self):
        """Build sections from the document tree, handling subsections and descriptions."""
        for node in self.doc_tree.traverse(nodes.section):
            subsections = list(node.traverse(nodes.section))

            if len(subsections) > 1:
                # get only the first section
                main_section = subsections[0]
                # remove subsections from the main section
                for subsection in main_section.traverse(nodes.section):
                    subsection.parent.remove(subsection)
                node = main_section

            section_title = node[0].astext()

            unwanted_types = (
                nodes.math,
                nodes.raw,
                nodes.image,
                nodes.figure,
                nodes.comment,
                nodes.literal_block,
            )

            # Collect all unwanted nodes first
            unwanted_nodes = [n for n in node.traverse() if isinstance(n, unwanted_types)]

            [n.parent.remove(n) for n in unwanted_nodes if n.parent]
            clean_text = node.astext()

            section_anchor_id = _title_to_anchor(section_title)
            self.sections.append(
                {
                    "title": section_title,
                    "text": clean_text,
                    "anchor_id": section_anchor_id,
                }
            )

    def generate_breadcrumbs(self, section_title: str) -> str:
        """
        Generate title breadcrumbs from the document structure.

        Parameters
        ----------
        section_title : str
            The section title to generate breadcrumbs for.

        Returns
        -------
        str
            Breadcrumb string.

        Notes
        -----
        The last part of `doc_name` is ignored because it represents the file name,
        which is not needed for the breadcrumb trail. The breadcrumb trail is generated
        by iterating over the parts of `doc_name` and fetching the title from the environment.
        """
        docs_parts = self.doc_name.split("/")[:-1]

        breadcrumb_parts = [
            self.env.titles[part].astext() for part in docs_parts if part in self.env.titles
        ]

        # Create breadcrumb hierarchy
        breadcrumbs = " > ".join(breadcrumb_parts)
        ignore_doc_title = section_title == self.doc_title

        # Construct final breadcrumb path
        if ignore_doc_title:
            return f"{breadcrumbs} > {section_title}" if breadcrumbs else section_title

        return (
            f"{breadcrumbs} > {self.doc_title} > {section_title}"
            if breadcrumbs
            else f"{self.doc_title} > {section_title}"
        )

    @property
    def indices(self):
        """Generate indices for each section."""
        for section in self.sections:
            breadcrumbs = self.generate_breadcrumbs(section["title"])
            self.object_id = filter_search_documents(
                self.filter_options, self.doc_name, self.parent_title
            )
            yield {
                "objectID": self.object_id,
                "href": f"{self.doc_path}#{section['anchor_id']}",
                "title": breadcrumbs,
                "text": section["text"],
            }


def _title_to_anchor(title: str) -> str:
    """Convert a title to a URL-friendly anchor identifier."""
    return re.sub(r"[^\w\s-]", "", title.lower().strip().replace(" ", "-"))


def filter_search_documents(filters, doc_name, doc_title):
    """Filter search documents based on the provided filters."""
    if not filters:
        return doc_title
    for display_doc_name, doc_names in filters.items():
        for value in doc_names:
            if doc_name.startswith(value.rstrip("/")):
                return display_doc_name
    return doc_title


def create_search_index(app, exception):
    """
    Generate search index at the end of the Sphinx build process.

    Parameters
    ----------
    app : Sphinx
        Sphinx application instance.
    exception : Exception
        Exception raised during the build process, if any.
    """
    if exception:
        return

    search_index_list = []

    static_search_options = app.config.html_theme_options.get("static_search", {})
    excluded_docs = static_search_options.get("files_to_exclude", [])
    included_docs = app.env.found_docs
    filter_options = app.config.html_theme_options.get("search_filters", {})

    patterns = app.env.config.index_patterns or {}
    if patterns:
        logger.error(
            "The 'index_patterns' is depreciated and no longer supported. "
            "Please see the documentation https://sphinxdocs.ansys.com for the "
            "new search configuration options."
        )

    patterns = app.env.config.index_patterns or {}
    if patterns:
        # Raise warning that this feature will deprecate in the future
        logger.warning(
            "The 'index_patterns' configuration option is deprecated and will be removed"
            " in v1.5.0 onwards. "
            "It will index all paragraphs, titles, and literals by default. "
            "More information can be found in the documentation at https://sphinxdocs.ansys.com.",
        )

    for exclude_doc in excluded_docs:
        exclude_doc = Path(exclude_doc).resolve()

        # Exclude documents based on whether exclude_doc is a folder or a file:
        # - For folders, exclude all documents within the folder.
        # - For files, exclude only the exact file match.
        included_docs = [
            doc for doc in included_docs if not Path(doc).resolve().is_relative_to(exclude_doc)
        ]

    for document in included_docs:
        search_index = SearchIndex(document, app, filter_options=filter_options)
        search_index.build_sections()
        search_index_list.extend(search_index.indices)

    search_index_path = Path(app.builder.outdir) / "_static" / "search.json"
    with search_index_path.open("w", encoding="utf-8") as index_file:
        json.dump(search_index_list, index_file, ensure_ascii=False, separators=(",", ":"))
